@@ -128,6 +128,7 @@
 #define RegionNull(r) REGION_NULL(NULL, r)
 #define RegionNotEmpty(r) REGION_NOTEMPTY(NULL, r)
 #define RegionEmpty(r) REGION_EMPTY(NULL, r)
+#define RegionEqual(a, b) REGION_EQUAL(NULL, a, b)
 #define RegionDestroy(r) REGION_DESTROY(NULL, r)
 #else
 #define region_from_bitmap BitmapToRegion
@@ -149,6 +150,10 @@ static inline void FreePixmap(PixmapPtr pixmap)
 }
 #endif
 
+#if !HAS_DIXREGISTERPRIVATEKEY
+#define dixPrivateKeyRegistered(key__) (*(key__) != 0)
+#endif
+
 #if XORG_VERSION_CURRENT >= XORG_VERSION_NUMERIC(1,9,99,902,0)
 #define SourceValidate(d, x, y, w, h, mode) \
 	if ((d)->pScreen->SourceValidate) (d)->pScreen->SourceValidate(d, x, y, w, h, mode)
@@ -157,4 +162,8 @@ static inline void FreePixmap(PixmapPtr pixmap)
 	if ((d)->pScreen->SourceValidate) (d)->pScreen->SourceValidate(d, x, y, w, h)
 #endif
 
+#if XORG_VERSION_CURRENT >= XORG_VERSION_NUMERIC(1,14,99,2,0)
+#define DamageUnregister(d, dd) DamageUnregister(dd)
+#endif
+
 #endif
