@@ -204,9 +204,17 @@ static const SymTabRec intel_chipsets[] = {
 	{PCI_CHIP_HASWELL_CRW_E_GT1,		"HD Graphics" }, /* ??? */
 	{PCI_CHIP_HASWELL_CRW_E_GT2,		"HD Graphics" }, /* ??? */
 	{PCI_CHIP_HASWELL_CRW_E_GT3,		"Iris(TM) Pro Graphics 5200" },
-	{-1,					NULL}
+
+	/* Valleyview (Baytail) */
+	{0x0f30, "HD Graphics"},
+	{0x0f31, "HD Graphics"},
+	{0x0f32, "HD Graphics"},
+	{0x0f33, "HD Graphics"},
+	{0x0155, "HD Graphics"},
+	{0x0157, "HD Graphics"},
+
+	{-1, NULL} /* Sentinel */
 };
-#define NUM_CHIPSETS (sizeof(intel_chipsets) / sizeof(intel_chipsets[0]))
 
 static const struct pci_id_match intel_device_match[] = {
 #if UMS
@@ -258,23 +266,30 @@ static const struct pci_id_match intel_device_match[] = {
 };
 
 void
-intel_detect_chipset(ScrnInfoPtr scrn,
-		     EntityInfoPtr ent,
-		     struct pci_device *pci)
+intel_detect_chipset(ScrnInfoPtr scrn, EntityInfoPtr ent)
 {
 	MessageType from = X_PROBED;
 	const char *name = NULL;
+	int devid;
 	int i;
 
 	if (ent->device->chipID >= 0) {
 		xf86DrvMsg(scrn->scrnIndex, from = X_CONFIG,
 			   "ChipID override: 0x%04X\n",
 			   ent->device->chipID);
-		pci->device_id = ent->device->chipID;
+		devid = ent->device->chipID;
+	} else {
+		struct pci_device *pci;
+
+		pci = xf86GetPciInfoForEntity(ent->index);
+		if (pci != NULL)
+			devid = pci->device_id;
+		else
+			devid = intel_get_device_id(scrn);
 	}
 
 	for (i = 0; intel_chipsets[i].name != NULL; i++) {
-		if (pci->device_id == intel_chipsets[i].token) {
+		if (devid == intel_chipsets[i].token) {
 			name = intel_chipsets[i].name;
 			break;
 		}
@@ -283,20 +298,21 @@ intel_detect_chipset(ScrnInfoPtr scrn,
 		int gen = 0;
 
 		for (i = 0; intel_device_match[i].device_id != 0; i++) {
-			if (pci->device_id == intel_device_match[i].device_id) {
+			if (devid == intel_device_match[i].device_id) {
 				const struct intel_device_info *info = (void *)intel_device_match[i].match_data;
 				gen = info->gen >> 3;
 				break;
 			}
 		}
 
-		if(gen) {
+		if (gen) {
 			xf86DrvMsg(scrn->scrnIndex, from,
 				   "gen%d engineering sample\n", gen);
 		} else {
 			xf86DrvMsg(scrn->scrnIndex, X_WARNING,
 				   "Unknown chipset\n");
 		}
+
 		name = "unknown";
 	} else {
 		xf86DrvMsg(scrn->scrnIndex, from,
@@ -446,6 +462,22 @@ intel_scrn_create(DriverPtr		driver,
 {
 	ScrnInfoPtr scrn;
 
+	if (match_data == 0) {
+		int devid = intel_entity_get_devid(entity_num), i;
+		if (devid == 0)
+			return FALSE;
+
+		for (i = 0; intel_device_match[i].device_id != 0; i++) {
+			if (devid == intel_device_match[i].device_id) {
+				match_data = (intptr_t)&intel_device_match[i];
+				break;
+			}
+		}
+
+		if (match_data == 0)
+			return FALSE;
+	}
+
 	scrn = xf86AllocateScreen(driver, flags);
 	if (scrn == NULL)
 		return FALSE;
@@ -522,11 +554,7 @@ intel_platform_probe(DriverPtr driver,
 {
 	unsigned scrn_flags = 0;
 
-	if (!dev->pdev)
-		return FALSE;
-
-	if (intel_open_device(entity_num, dev->pdev,
-			      xf86_get_platform_device_attrib(dev, ODEV_ATTRIB_PATH)) == -1)
+	if (intel_open_device(entity_num, dev->pdev, dev) == -1)
 		return FALSE;
 
 	/* Allow ourselves to act as a slaved output if not primary */
